from django.core.exceptions import ValidationError
from django.test import TestCase

from system_maintenance.models import (
    DocumentationRecord, Hardware, MaintenanceRecord,
    MaintenanceRecordRelationship, MaintenanceType, Software, SysAdmin, System)
from system_maintenance.tests.utilities import populate_test_db


class MaintenanceRecordRelationshipTest(TestCase):

    def setUp(self):
        self.db_objects = populate_test_db()

    def test_cannot_save_relationship_to_self(self):
        record = self.db_objects['maintenance_record_1']
        relationship = MaintenanceRecordRelationship(
            referenced_record=record,
            referencing_record=record,
        )

        with self.assertRaises(ValidationError):
            relationship.save()
            relationship.full_clean()


class SaveAndRetrieveTests(TestCase):

    """
    Test that objects created by `populate_test_db()` can be retrieved from
    the database.
    """

    def setUp(self):
        self.db_objects = populate_test_db()

    def test_save_and_retrieve_documentation_records(self):
        records = DocumentationRecord.objects.all()

        self.assertEqual(records.count(), 2)
        self.assertEqual(records[0], self.db_objects['documentation_record_1'])
        self.assertEqual(records[1], self.db_objects['documentation_record_2'])

    def test_save_and_retrieve_hardwares(self):
        hardwares = Hardware.objects.all()

        self.assertEqual(hardwares.count(), 1)
        self.assertEqual(hardwares[0], self.db_objects['hardware'])

    def test_save_and_retrieve_maintenance_records(self):
        # Maintenance Records are in reverse chronological order
        records = MaintenanceRecord.objects.all()

        self.assertEqual(records.count(), 3)
        self.assertEqual(records[0], self.db_objects['maintenance_record_3'])
        self.assertEqual(records[1], self.db_objects['maintenance_record_2'])
        self.assertEqual(records[2], self.db_objects['maintenance_record_1'])

    def test_save_and_retrieve_maintenance_record_relationships(self):
        relationships = MaintenanceRecordRelationship.objects.all()

        self.assertEqual(relationships.count(), 1)
        self.assertEqual(
            relationships[0].referenced_record,
            self.db_objects['maintenance_record_1'])
        self.assertEqual(
            relationships[0].referencing_record,
            self.db_objects['maintenance_record_2'])

    def test_save_and_retrieve_maintenance_types(self):
        maintenance_types = MaintenanceType.objects.all()

        self.assertEqual(maintenance_types.count(), 2)
        self.assertEqual(
            maintenance_types[0], self.db_objects['maintenance_type_1'])
        self.assertEqual(
            maintenance_types[1], self.db_objects['maintenance_type_2'])

    def test_save_and_retrieve_softwares(self):
        softwares = Software.objects.all()

        self.assertEqual(softwares.count(), 1)
        self.assertEqual(softwares[0], self.db_objects['software'])

    def test_save_and_retrieve_sysadmins(self):
        sysadmins = SysAdmin.objects.all()

        self.assertEqual(sysadmins.count(), 2)
        self.assertEqual(sysadmins[0], self.db_objects['sysadmin'])
        self.assertEqual(sysadmins[1], self.db_objects['supersysadmin'])

    def test_save_and_retrieve_systems(self):
        systems = System.objects.all()

        self.assertEqual(systems.count(), 1)
        self.assertEqual(systems[0], self.db_objects['system'])



from django.contrib.auth.models import User
from django.urls import reverse

from system_maintenance.models import (
    DocumentationRecord, Hardware, MaintenanceRecord,
    MaintenanceRecordRelationship, MaintenanceType, Software, SysAdmin, System)


class CustomAssertions:

    """
    A mixin consisting of custom assertions.

    Use with a `TestCase` like this:

        from system_maintenance.tests.utilities import CustomAssertions

        class SomeTest(TestCase, CustomAssertions):
            ...
    """

    def assertRedirectUserToAuthentication(self, namespace):
        """
        Take a namespace, get the corresponding URL, and test whether
        it redirects to the System Maintenance authentication page
        (with the original URL as the 'next' page).
        """
        auth_url = reverse('system_maintenance:authentication')
        url = reverse(namespace)
        response = self.client.get(url)
        self.assertRedirects(response, '{}?next={}'.format(auth_url, url))


def populate_test_db():
    """
    Add records to an empty test database.
    Returns a dict of the saved objects.
    """
    User.objects.create_user(username='nonsysadmin', password='nonsysadmin')

    sysadmin_user = User.objects.create_user(
        username='sysadmin', password='sysadmin')
    sysadmin = SysAdmin.objects.create(user=sysadmin_user)

    supersysadmin_user = User.objects.create_superuser(
        username='supersysadmin', password='supersysadmin',
        email='supersysadmin@supersysadmin')
    supersysadmin = SysAdmin.objects.create(user=supersysadmin_user)

    maintenance_type_1 = MaintenanceType.objects.create(
        maintenance_type='Maintenance Type 1')
    maintenance_type_2 = MaintenanceType.objects.create(
        maintenance_type='Maintenance Type 2')

    documentation_record_1 = DocumentationRecord.objects.create(
        title='Documentation 1',
        maintenance_type=maintenance_type_1,
        documentation='',
    )
    documentation_record_2 = DocumentationRecord.objects.create(
        title='Documentation 2',
        maintenance_type=maintenance_type_2,
        documentation='',
    )

    hardware = Hardware.objects.create(name='Hardware 1')
    software = Software.objects.create(name='Software 1')
    system = System.objects.create(name='System 1')

    maintenance_record_1 = MaintenanceRecord.objects.create(
        system=system,
        sys_admin=sysadmin,
        maintenance_type=maintenance_type_1,
        description='',
        procedure='',
        problems='',
    )
    maintenance_record_1.hardware.add(hardware)

    maintenance_record_2 = MaintenanceRecord.objects.create(
        system=system,
        sys_admin=supersysadmin,
        maintenance_type=maintenance_type_2,
        description='',
        procedure='',
        problems='',
        status='Complete',
    )
    maintenance_record_2.software.add(software)
    maintenance_record_2.documentation_records.add(documentation_record_1)
    maintenance_record_2.documentation_records.add(documentation_record_2)

    MaintenanceRecordRelationship.objects.create(
        referenced_record=maintenance_record_1,
        referencing_record=maintenance_record_2,
    )

    maintenance_record_3 = MaintenanceRecord.objects.create(
        system=system,
        sys_admin=sysadmin,
        maintenance_type=maintenance_type_1,
        status='Failed',
    )
    maintenance_record_3.hardware.add(hardware)
    maintenance_record_3.software.add(software)

    db_objects = {
        'documentation_record_1': documentation_record_1,
        'documentation_record_2': documentation_record_2,
        'hardware': hardware,
        'maintenance_record_1': maintenance_record_1,
        'maintenance_record_2': maintenance_record_2,
        'maintenance_record_3': maintenance_record_3,
        'maintenance_type_1': maintenance_type_1,
        'maintenance_type_2': maintenance_type_2,
        'software': software,
        'supersysadmin': supersysadmin,
        'system': system,
        'sysadmin': sysadmin,
    }

    return db_objects


def login_normal_user(self):
    """
    Login as a normal user.
    """
    self.client.login(username='nonsysadmin', password='nonsysadmin')
    return self


def login_sysadmin_user(self):
    """
    Login as a sysadmin user.
    """
    self.client.login(username='sysadmin', password='sysadmin')
    return self


def login_sysadmin_superuser(self):
    """
    Login as a sysadmin superuser.
    """
    self.client.login(username='supersysadmin', password='supersysadmin')
    return self



import re

from django.contrib.auth import views as auth_views
from django.urls import resolve, reverse
from django.template.loader import render_to_string
from django.test import TestCase

from system_maintenance import views
from system_maintenance.models import DocumentationRecord, MaintenanceRecord
from system_maintenance.tests.utilities import (
    CustomAssertions, login_normal_user, login_sysadmin_superuser,
    login_sysadmin_user, populate_test_db)


class CommonViewTests:

    """
    A mixin consisting of a set of tests that are common to multiple views.

    Unless these tests are overridden, a `TestCase` that uses this mixin must
    define a set of view-specific variables in the `setUp()` method similar
    to this:

        def setUp(self):
            populate_test_db()
            login_sysadmin_user(self)

            self.namespace = 'system_maintenance:documentation_record_list'
            self.template = 'system_maintenance/documentation_record_list.html'
            self.title = 'Documentation Records'
            self.url = '/system_maintenance/documentation/'
            self.view = views.DocumentationRecordListView

            self.get_response()
    """

    def get_response(self):
        """
        Get response based on URL.
        """
        self.response = self.client.get(self.url)

    def test_namespace_reverses_to_url(self):
        """
        Test that a given namespace reverses to the correct URL.
        """
        self.assertEqual(reverse(self.namespace), self.url)

    def test_url_resolves_to_view(self):
        """
        Test that a given URL resolves to the correct view.
        """
        found = resolve(self.url)
        found_name = found.func.__name__
        self.assertEqual(found_name, getattr(self.view, '__name__', None))

    def test_view_uses_correct_template(self):
        """
        Test that view's response uses the correct template
        """
        self.assertTemplateUsed(self.response, self.template)

    def test_view_returns_correct_title(self):
        """
        Test that view's response contains the correct page title.
        """
        self.assertContains(
            self.response, '<title>{}</title>'.format(self.title))

    def test_view_returns_static_files_links(self):
        """
        Test that view's response contains links to the correct static files.
        """
        static_files = [
            'bootstrap.min.css',
            'bootstrap.min.js',
            'jquery.min.js',
            '/static/system_maintenance/css/app.css',
            '/static/system_maintenance/js/app.js',
        ]
        for static in static_files:
            self.assertContains(self.response, static)

    def test_view_returns_correct_html(self):
        """
        Test that a view's response contains the correct HTML based on the
        template.
        """
        self.assertEqual(self.response.status_code, 200)

        # CSRF tokens don't get render_to_string'd
        csrf_regex = r'<input[^>]+csrfmiddlewaretoken[^>]+>'
        observed_html = re.sub(csrf_regex, '', self.response.content.decode())

        try:
            context = self.context
        except AttributeError:
            context = {}

        expected_html = render_to_string(self.template, context)

        self.assertEqual(observed_html, expected_html)


class RedirectAnonymousUserToAuthenticationTest(TestCase, CustomAssertions):

    def test_system_maintenance_home_view(self):
        self.assertRedirectUserToAuthentication(
            'system_maintenance:system_maintenance_home_view')

    def test_documentation_record_list(self):
        self.assertRedirectUserToAuthentication(
            'system_maintenance:documentation_record_list')


class RedirectNonSysAdminUserToAuthenticationTest(TestCase, CustomAssertions):

    def setUp(self):
        populate_test_db()
        login_normal_user(self)

    def test_system_maintenance_home_view(self):
        self.assertRedirectUserToAuthentication(
            'system_maintenance:system_maintenance_home_view')

    def test_documentation_record_list(self):
        self.assertRedirectUserToAuthentication(
            'system_maintenance:documentation_record_list')


class AuthenticationViewTest(TestCase, CommonViewTests):

    def setUp(self):
        self.namespace = 'system_maintenance:authentication'
        self.template = 'system_maintenance/authentication.html'
        self.title = 'System Maintenance'
        self.url = '/system_maintenance/authentication/'

        self.get_response()

    def test_url_resolves_to_view(self):
        found = resolve(self.url)
        self.assertEqual(
            found.func.view_class().template_name,
            auth_views.LoginView.as_view().view_class().template_name)


class HomeViewTest(TestCase, CommonViewTests):

    def setUp(self):
        populate_test_db()
        login_sysadmin_user(self)

        self.namespace = 'system_maintenance:system_maintenance_home_view'
        self.template = 'system_maintenance/system_maintenance_home.html'
        self.title = 'System Maintenance'
        self.url = '/system_maintenance/'

        self.get_response()

    def test_url_resolves_to_view(self):
        found = resolve(self.url)
        self.assertEqual(found.func, views.system_maintenance_home_view)

    def test_view_returns_correct_html(self):
        """
        Test that normal system administrators have links to System
        Maintenance pages, but not admin pages.
        """
        self.assertEqual(self.response.status_code, 200)

        app_urls = [
            'records/',
            'documentation/',
        ]
        for url in app_urls:
            self.assertContains(
                self.response, '/system_maintenance/{}'.format(url))

        self.assertContains(
            self.response, 'System maintenance records and other important ' +
            'system administration information is accessible via the ' +
            'buttons below.')

        unexpected_superuser_only_content = [
            'System Maintenance Admin Page',
            '/admin/system_maintenance/',
            'glyphicon-wrench',
        ]
        for content in unexpected_superuser_only_content:
            self.assertNotContains(self.response, content)

    def test_view_returns_correct_html_for_superuser_sysadmin(self):
        """
        Test that superuser system administrators have links to System
        Maintenance admin pages.
        """
        login_sysadmin_superuser(self)
        self.get_response()
        self.assertEqual(self.response.status_code, 200)

        model_names = [
            'documentationrecord',
            'hardware',
            'maintenancerecord',
            'maintenancetype',
            'software',
            'sysadmin',
            'system',
        ]
        for model in model_names:
            self.assertContains(
                self.response, '/admin/system_maintenance/{}/'.format(model))

        self.assertContains(self.response, 'System Maintenance Admin Page')
        self.assertContains(self.response, 'glyphicon-wrench')


class DocumentationRecordListViewTest(TestCase, CommonViewTests):

    def setUp(self):
        populate_test_db()
        login_sysadmin_user(self)

        self.namespace = 'system_maintenance:documentation_record_list'
        self.template = 'system_maintenance/documentation_record_list.html'
        self.title = 'Documentation Records'
        self.url = '/system_maintenance/documentation/'
        self.view = views.DocumentationRecordListView

        self.get_response()

        self.context = {
            'object_list': DocumentationRecord.objects.all(),
        }


class MaintenanceRecordListViewTest(TestCase, CommonViewTests):

    def setUp(self):
        populate_test_db()
        login_sysadmin_user(self)

        self.namespace = 'system_maintenance:maintenance_record_list'
        self.template = 'system_maintenance/maintenance_record_list.html'
        self.title = 'Maintenance Records'
        self.url = '/system_maintenance/records/'
        self.view = views.MaintenanceRecordListView

        self.get_response()

        self.context = {
            'object_list': MaintenanceRecord.objects.all(),
        }



# Generated by Django 2.0.9 on 2018-11-05 10:44

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import markupfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentationRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='Enter a brief, descriptive title for this documentation.', max_length=255, unique=True)),
                ('documentation', markupfield.fields.MarkupField(blank=True, help_text='Document how to perform a task.', null=True, rendered_field=True)),
                ('documentation_markup_type', models.CharField(choices=[('', '--'), ('Markdown', 'Markdown'), ('Markdown Basic', 'Markdown Basic'), ('Plain Text', 'Plain Text'), ('reStructuredText', 'reStructuredText')], default='Markdown', max_length=30)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('_documentation_rendered', models.TextField(editable=False, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ['title'],
            },
        ),
        migrations.CreateModel(
            name='Hardware',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Enter the type of hardware.', max_length=255, unique=True)),
            ],
            options={
                'verbose_name_plural': 'hardware',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='MaintenanceRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', markupfield.fields.MarkupField(blank=True, help_text='Enter a description of the system maintenance performed.', null=True, rendered_field=True)),
                ('description_markup_type', models.CharField(choices=[('', '--'), ('Markdown', 'Markdown'), ('Markdown Basic', 'Markdown Basic'), ('Plain Text', 'Plain Text'), ('reStructuredText', 'reStructuredText')], default='Markdown', max_length=30)),
                ('procedure', markupfield.fields.MarkupField(blank=True, help_text='Enter details of how the system maintenance was performed.', null=True, rendered_field=True)),
                ('_description_rendered', models.TextField(editable=False, null=True)),
                ('procedure_markup_type', models.CharField(choices=[('', '--'), ('Markdown', 'Markdown'), ('Markdown Basic', 'Markdown Basic'), ('Plain Text', 'Plain Text'), ('reStructuredText', 'reStructuredText')], default='Markdown', max_length=30)),
                ('problems', markupfield.fields.MarkupField(blank=True, help_text='Describe problems that arose during system maintenance.', null=True, rendered_field=True)),
                ('_procedure_rendered', models.TextField(editable=False, null=True)),
                ('problems_markup_type', models.CharField(choices=[('', '--'), ('Markdown', 'Markdown'), ('Markdown Basic', 'Markdown Basic'), ('Plain Text', 'Plain Text'), ('reStructuredText', 'reStructuredText')], default='Markdown', max_length=30)),
                ('_problems_rendered', models.TextField(editable=False, null=True)),
                ('status', models.CharField(choices=[('Complete', 'Complete'), ('In Progress', 'In Progress'), ('Failed', 'Failed')], default='In Progress', help_text='What is the current status of the system maintenance?', max_length=15)),
                ('datetime', models.DateTimeField(default=django.utils.timezone.now, help_text='Specify the date/time that the system maintenance was performed.', verbose_name='maintenance date/time')),
                ('documentation_records', models.ManyToManyField(blank=True, help_text='Select documentation relevant to this system maintenance.<br>', related_name='maintenance_records', to='system_maintenance.DocumentationRecord')),
                ('hardware', models.ManyToManyField(blank=True, help_text='Select the hardware involved in the system maintenance.', to='system_maintenance.Hardware')),
            ],
            options={
                'verbose_name': 'maintenance record',
                'verbose_name_plural': 'maintenance records',
                'ordering': ['-datetime'],
            },
        ),
        migrations.CreateModel(
            name='MaintenanceRecordRelationship',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('referenced_record', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='referenced_record', to='system_maintenance.MaintenanceRecord')),
                ('referencing_record', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='referencing_record', to='system_maintenance.MaintenanceRecord')),
            ],
            options={
                'ordering': ['referencing_record'],
            },
        ),
        migrations.CreateModel(
            name='MaintenanceType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('maintenance_type', models.CharField(help_text="Enter a type of maintenance (e.g., 'Software Installation').", max_length=255, unique=True)),
                ('description', models.TextField(blank=True, help_text='Enter a description of the maintenance type.')),
            ],
            options={
                'ordering': ['maintenance_type'],
            },
        ),
        migrations.CreateModel(
            name='Software',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text="Enter the software's name.", max_length=255, unique=True)),
            ],
            options={
                'verbose_name': 'software package',
                'verbose_name_plural': 'software packages',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='SysAdmin',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('user', models.OneToOneField(help_text='Select a user.', on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'system administrator',
                'verbose_name_plural': 'system administrators',
            },
        ),
        migrations.CreateModel(
            name='System',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Enter a brief, unique identifier for the system.', max_length=255, unique=True, verbose_name='system name')),
                ('description', models.TextField(blank=True, help_text='Enter a description of the system.')),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.AddField(
            model_name='maintenancerecord',
            name='maintenance_type',
            field=models.ForeignKey(help_text='Select/Create a maintenance type.', on_delete=django.db.models.deletion.PROTECT, to='system_maintenance.MaintenanceType'),
        ),
        migrations.AddField(
            model_name='maintenancerecord',
            name='referenced_records',
            field=models.ManyToManyField(related_name='referencing_records', through='system_maintenance.MaintenanceRecordRelationship', to='system_maintenance.MaintenanceRecord'),
        ),
        migrations.AddField(
            model_name='maintenancerecord',
            name='software',
            field=models.ManyToManyField(blank=True, help_text='Select the software involved in the system maintenance.', to='system_maintenance.Software'),
        ),
        migrations.AddField(
            model_name='maintenancerecord',
            name='sys_admin',
            field=models.ForeignKey(help_text='Select a system administrator.', on_delete=django.db.models.deletion.PROTECT, to='system_maintenance.SysAdmin'),
        ),
        migrations.AddField(
            model_name='maintenancerecord',
            name='system',
            field=models.ForeignKey(help_text='Select/Create a system.', on_delete=django.db.models.deletion.PROTECT, to='system_maintenance.System'),
        ),
        migrations.AddField(
            model_name='documentationrecord',
            name='maintenance_type',
            field=models.ForeignKey(help_text='Select/Create a maintenance type.', on_delete=django.db.models.deletion.PROTECT, to='system_maintenance.MaintenanceType'),
        ),
        migrations.AlterUniqueTogether(
            name='maintenancerecordrelationship',
            unique_together={('referencing_record', 'referenced_record')},
        ),
    ]



from selenium.webdriver.support.color import Color

from .base import FunctionalTest


class LayoutAndStylingTest(FunctionalTest):

    def test_authentication(self):
        # Go to the authentication page
        self.browser.get(self.system_maintenance_url('authentication'))
        window_width = 768
        self.browser.set_window_size(window_width, window_width / 2)

        # Username and password input boxes are centered
        self.find_authentication_elements()
        center_username = self.username_inputbox.location['x'] + \
            self.username_inputbox.size['width'] / 2
        center_password = self.password_inputbox.location['x'] + \
            self.password_inputbox.size['width'] / 2
        self.assertAlmostEqual(center_username, window_width / 2, delta=5)
        self.assertAlmostEqual(center_password, window_width / 2, delta=5)

    def test_maintenance_record_list(self):
        # Sign in as sysadmin and go to Maintenance Records
        self.browser.get(self.system_maintenance_url('records'))
        self.login_as('sysadmin')

        # See, based on the color-coded backgrounds, that the status of the
        # first record is 'Failed', the second is 'Complete', and the third is
        # 'In Progress'
        list_group_items = self.browser.find_elements_by_class_name(
            'list-group-item')
        background_colors = ['#f2dede', '#ffffff', '#fcf8e3']

        self.assertEqual(len(list_group_items), 3)
        for item, color in zip(list_group_items, background_colors):
            self.assertEqual(Color.from_string(
                item.value_of_css_property('background-color')).hex, color)



from .base import FunctionalTest


class AuthenticationTest(FunctionalTest):

    def test_anon_user_redirected_to_authentication(self):
        # Try to go to the System Maintenance homepage
        self.browser.get(self.system_maintenance_url())

        # Not logged in, so get redirected to the SysAdmin Authentication page
        self.assertIn('System Maintenance', self.browser.title)
        header_text = self.browser.find_element_by_tag_name('h1').text
        self.assertIn('SysAdmin Authentication', header_text)

        # See input boxes for username & password
        self.find_authentication_elements()

        self.assertEqual(
            self.username_inputbox.get_attribute('placeholder'),
            'Enter username')

        self.assertEqual(
            self.password_inputbox.get_attribute('placeholder'),
            'Enter password')

        self.assertEqual(self.login_button.text, 'Login')

    def test_username_and_password_required(self):
        # Go to the authentication page
        self.browser.get(self.system_maintenance_url('authentication'))

        # Accidentally click 'Login' button without entering credentials
        self.find_authentication_elements()
        self.login_button.click()

        # See two error messages about required fields
        form_errors = self.browser.find_elements_by_class_name('has-error')
        self.assertEqual(len(form_errors), 2)
        for error in form_errors:
            self.assertIn('This field is required.', error.text)

    def test_incorrect_credentials_raise_error(self):
        # Go to the authentication page
        self.browser.get(self.system_maintenance_url('authentication'))

        # Enter incorrect credentials
        self.login_as('nobody')

        # See error message about entering correct username and password
        self.assertIn(
            'Please enter a correct username and password. Note that both ' +
            'fields may be case-sensitive.',
            self.browser.find_element_by_class_name('alert-danger').text)

    def test_nonsysadmin_is_denied_access_and_can_logout(self):
        # Go to the authentication page
        self.browser.get(self.system_maintenance_url('authentication'))

        # Enter non-sysadmin credentials
        self.login_as('nonsysadmin')

        # See 'Access denied.' message about not being a sys admin
        self.assertIn(
            'Access denied.', self.browser.find_element_by_tag_name('h1').text)
        self.assertIn(
            'Hello nonsysadmin. You are not a system administrator.',
            self.browser.find_element_by_tag_name('p').text)

        # Click 'Logout' button and get redirected to authentication page.
        self.logout_button = self.browser.find_element_by_tag_name('a')
        self.logout_button.click()

    def test_can_login_as_sysadmin_and_can_logout(self):
        # Go to the authentication page
        self.browser.get(self.system_maintenance_url('authentication'))

        # Enter sysadmin credentials
        self.login_as('sysadmin')

        # Check that redirected to System Maintenance home page
        self.assertIn('System Maintenance', self.browser.title)
        self.assertEqual(
            self.browser.find_element_by_tag_name('p').text,
            'System maintenance records and other important system '
            'administration information is accessible via the buttons below.')

        self.assertEqual(
            len(self.browser.find_elements_by_css_selector(
                '.btn.full-width-on-mobile')), 7)
        self.assertEqual(
            len(self.browser.find_elements_by_css_selector(
                '.btn-group.hide-on-mobile > .btn')), 7)

        # Logout and get redirected to SysAdmin Authentication page
        self.browser.get(self.system_maintenance_url('logout'))
        header_text = self.browser.find_element_by_tag_name('h1').text
        self.assertIn('SysAdmin Authentication', header_text)

    def test_can_login_as_super_sysadmin(self):
        # Go to the authentication page
        self.browser.get(self.system_maintenance_url('authentication'))

        # Enter superuser sysadmin credentials
        self.login_as('supersysadmin')

        # Check that redirected to System Maintenance home page w/ admin access
        self.assertIn('System Maintenance', self.browser.title)
        paragraphs = self.browser.find_elements_by_tag_name('p')
        self.assertIn('System maintenance records and ', paragraphs[0].text)
        self.assertIn('To add or change system ', paragraphs[1].text)

        self.assertEqual(
            len(self.browser.find_elements_by_css_selector(
                '.btn.full-width-on-mobile')), 7)
        self.assertEqual(
            len(self.browser.find_elements_by_css_selector(
                '.btn-group.hide-on-mobile > .btn')), 14)



from django.urls import path
from django.contrib.auth import views as auth_views

from . import views


app_name = 'system_maintenance'

urlpatterns = [
    path('', views.system_maintenance_home_view, name='system_maintenance_home_view'),
    path('authentication/', auth_views.LoginView.as_view(template_name='system_maintenance/authentication.html'), name='authentication'),
    path('documentation/', views.DocumentationRecordListView.as_view(), name='documentation_record_list'),
    path('documentation/<int:pk>/', views.DocumentationRecordDetailView.as_view(), name='documentation_record_detail'),
    path('logout/', auth_views.LogoutView.as_view(next_page='/system_maintenance/'), name='logout'),
    path('raw/<type_of_record>/<type_of_field>/<int:record_pk>/', views.raw_view, name='raw_view'),
    path('records/', views.MaintenanceRecordListView.as_view(), name='maintenance_record_list'),
    path('records/<int:pk>/', views.MaintenanceRecordDetailView.as_view(), name='maintenance_record_detail'),
]



from django.contrib.staticfiles.testing import StaticLiveServerTestCase

from selenium import webdriver
from selenium.webdriver.common.keys import Keys

from system_maintenance.tests.utilities import populate_test_db


class FunctionalTest(StaticLiveServerTestCase):

    def setUp(self):
        populate_test_db()

        self.browser = webdriver.Firefox()
        self.browser.implicitly_wait(3)

        self.username_inputbox = None
        self.password_inputbox = None
        self.login_button = None

    def tearDown(self):
        self.browser.quit()

    def find_authentication_elements(self):
        self.username_inputbox = self.browser.find_element_by_id('id_username')
        self.password_inputbox = self.browser.find_element_by_id('id_password')
        self.login_button = self.browser.find_element_by_tag_name('button')

    def login_as(self, username):
        self.find_authentication_elements()
        self.username_inputbox.send_keys(username)
        self.password_inputbox.send_keys(username)
        self.login_button.click()

    def system_maintenance_url(self, url_stem=''):
        return '{}/system_maintenance/{}'.format(
            self.live_server_url, url_stem)
